package hungerhelper.restaurants;

import algs12.Point2D;
import algs24.MaxPQ;
import algs24.MinPQ;

import java.util.*;

public class Restaurants {
    private final HashMap<RestaurantKey, Restaurant> restaurants = new HashMap<>();

    public void add(Restaurant restaurant) {
        restaurants.put(
            new RestaurantKey(restaurant.getName(), restaurant.getLocation()),
            restaurant
        );
    }

    public Restaurant get(RestaurantKey key) throws Exception {
        return restaurants.get(key);
    }

    public Iterable<Restaurant> get(String name) {
        HashSet<Restaurant> result = new HashSet<>();
        for(RestaurantKey key : restaurants.keySet()) {
            if(key.getName().equals(name)) {
                result.add(restaurants.get(key));
            }
        }
        return Collections.unmodifiableSet(result);
    }

    // Return n restaurants of a certain cuisine, with the closest restaurants
    // earlier in the list
    private static class RestDist implements Comparable<RestDist> {
        Restaurant r;
        double d;

        @Override
        public int compareTo(RestDist o) {
            return Double.compare(d, o.d);
        }
    }
    public ArrayList<Restaurant> getNearby(Point2D location, int limit) {
        MaxPQ<RestDist> pq = new MaxPQ<>();
        for(Restaurant r : restaurants.values()) {
            double dist = r.getLocation().distanceTo(location);
            RestDist rd = new RestDist();
            rd.d = dist;
            rd.r = r;
            pq.insert(rd);
            if(pq.size() == limit + 1) {
                pq.delMax();
            }
        }
        ArrayList<Restaurant> result = new ArrayList<>();
        for(RestDist rd : pq) {
            result.add(rd.r);
        }
        return new ArrayList<Restaurant>(result.reversed());
    }
}
