package stdlib;

/**
 * An implementation of a hash table where each value has a composite key
 * generated from 2 objects.
 *
 * Acts like java.util.HashSet, but makes additional assumptions of clients:
 * <ul>
 * <li>No key may be null.</li>
 * <li>No key may be put more than once (no updates).</li>
 * <li>It uses == rather than .equals to compare objects for equality.</li>
 * </ul>
 */
public class YHashSet2<K1,K2> {
	private YHashMap2<K1, K2, Object> map = new YHashMap2<> ();
	private static final Object value = new Object ();

	public void put (K1 key1, K2 key2) {
		map.put (key1, key2, value);
	}
	public boolean contains (K1 key1, K2 key2) {
		return map.get (key1, key2) == value;
	}
}
