package stdlib;

/**
 * An implementation of a hash table where each value has a composite key
 * generated from 3 objects.
 *
 * Acts like java.util.HashSet, but makes additional assumptions of clients:
 * <ul>
 * <li>No key may be null.</li>
 * <li>No key may be put more than once (no updates).</li>
 * <li>It uses == rather than .equals to compare objects for equality.</li>
 * </ul>
 */
public class YHashSet3<K1,K2,K3> {
	private YHashMap3<K1, K2, K3, Object> map = new YHashMap3<> ();
	private static final Object value = new Object ();

	public void put (K1 key1, K2 key2, K3 key3) {
		map.put (key1, key2, key3, value);
	}
	public boolean contains (K1 key1, K2 key2, K3 key3) {
		return map.get (key1, key2, key3) == value;
	}
}
