package stdlib;
//- LJVExample.java --- Some example visualizations

//- Author:     John Hamer <J.Hamer@cs.auckland.ac.nz>
//- Created:    Mon Jul 12 15:24:19 2004
//- Time-stamp: <2004-08-23 12:47:15 jham005>

//- Copyright (C) 2004  John Hamer, University of Auckland
//-
//-   This program is free software; you can redistribute it and/or
//-   modify it under the terms of the GNU General Public License
//-   as published by the Free Software Foundation; either version 2
//-   of the License, or (at your option) any later version.
//-
//-   This program is distributed in the hope that it will be useful,
//-   but WITHOUT ANY WARRANTY; without even the implied warranty of
//-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//-   GNU General Public License for more details.
//-
//-   You should have received a copy of the GNU General Public License along
//-   with this program; if not, write to the Free Software Foundation, Inc.,
//-   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

//- $Id: LJVExample.java,v 1.2 2004/07/14 02:14:14 jham005 Exp $

import java.util.ArrayList;

@SuppressWarnings({ "rawtypes", "unchecked" })
class YLJVExample {

	public static void main( String[] args ) {
		YLJV.resetVeryVerbose();


		{//0
			//- String is not a primitive type
			YLJV.drawGraph( "Hello" );
		}

		{//1
			//- Object arrays hold references; primitive arrays hold values
			Object x = new Object[]{ new String[]{ "a", "b", "c" }, new int[]{ 1, 2, 3 }, Integer.valueOf(42), Double.valueOf(3.2) };
			YLJV.drawGraph( x );
		}

		{//2
			//- Assignment does not create a new object
			String x = "Hello";
			String y = x;
			YLJV.drawGraph( new Object[]{ x, y } );
		}

		{//3
			String x = "Hello";
			String y = new String(x);
			YLJV.drawGraph( new Object[]{ x, y } );
		}

		{//4
			//- How multi-dimensional arrays are represented in Java.
			YLJV.drawGraph( new int[4][5] );
		}

		{//5
			//- Cyclical structures, classes (with and) without a toString.
			Node n  = new Node( "top",   2 );
			n.left  = new Node( "left",  1 );
			n.right = new Node( "right", 1 );
			n.right.left  = n;
			n.right.right = n;

			YLJV.resetVerbose( )
			.setFieldAttribute( "left",  "color=red,fontcolor=red"  )
			.setFieldAttribute( "right", "color=blue,fontcolor=blue" )
			.setClassAttribute( Node.class, "color=pink,style=filled" )
			.ignoreField( "level" )
			.ignoreField( "ok" )
			.treatClassAsPrimitive( String.class )
			.hideFieldNamesInLabels( );

			YLJV.drawGraph( n );
		}

		{//6
			//- Paul's example
			YLJV.resetVerbose( )
			//.treatAsPrimitive( String.class );
			.ignoreField( "hash" ).ignoreField( "hash32" );

			ArrayList a = new ArrayList();
			a.add(new Person("Albert", true, 35));
			a.add(new Person("Betty", false, 20));
			a.add(new java.awt.Point(100, -100));
			YLJV.drawGraph( a );
		}
		{
			YLJV.resetVerbose ()
			.hideFieldNamesInLabels ()
			.ignoreField ("N")
			.ignoreField ("H")
			.ignoreField ("val");
			//LJV.drawGraph (st);
		}
	}

	static class Node {
		String name;
		int level;
		boolean ok;
		Node left, right;
		public Node( String n, int l ) {
			name = n;
			level = l;
			ok = l%2 == 0;
		}
		public String toString( ) { return ""; }
	}

	static class Person {
		private String name;
		private boolean isMale;
		private int age;
		public Person(String n, boolean m, int a) {
			name = n;
			isMale = m;
			age = a;
		}
	}
}
