package stdlib;

import java.util.Arrays;
import java.util.Collection;
import java.util.concurrent.*;
import java.util.function.Consumer;
import java.util.function.Function;

/**
 * @param <TE> e is professor object
 * @param <TA> a is student object
 * @param <TG> g is input to generate professor and student object (using g2e and g2a)
 * @param <TR> r is result
 */
public abstract class YTest<TE, TA, TG, TR> {
	public static boolean DEBUG = false;
	public static double[] doubleArrayCopy (double[] a) {
		double[] b = new double[a.length];
		System.arraycopy (a, 0, b, 0, a.length);
		return b;
	}
	public static String stringOf (double[] a) {
		StringBuffer sb = new StringBuffer ();
		for (double d : a) {
			sb.append (d);
			sb.append (" ");
		}
		return sb.toString ();
	}
	public static String randomDoubleStringProbablyUniqueElements (int minLength, int maxLength) {
		return stringOf (randomDoubleArrayProbablyUniqueElements (minLength, maxLength));
	}
	public static String randomDoubleStringUniqueElements (int minLength, int maxLength, int minVal, int maxVal) {
		return stringOf (randomDoubleArrayUniqueElements (minLength, maxLength, minVal, maxVal));
	}
	public static String randomDoubleStringWithLikelyItem (double item, double probabilityOfItem, int minLength, int maxLength, int minVal, int maxVal) {
		return stringOf (randomDoubleArrayWithLikelyItem (item, probabilityOfItem, minLength, maxLength, minVal, maxVal));
	}
	public static String randomDoubleString (int minLength, int maxLength, int minVal, int maxVal) {
		return stringOf (randomDoubleArray (minLength, maxLength, minVal, maxVal));
	}
	public static String randomDoubleStringSorted (int minLength, int maxLength, int minVal, int maxVal) {
		return stringOf (randomDoubleArraySorted (minLength, maxLength, minVal, maxVal));
	}
	
	public static double[] randomDoubleArrayProbablyUniqueElements (int minLength, int maxLength) {
		int length = StdRandom.uniform (minLength, maxLength);
		double[] result = new double[length];
		for (int i=0; i<length; i++) {
			result[i] = StdRandom.uniform (-100_000,100_000)/100.0;
		}
		return result;
	}
	public static double[] randomDoubleArrayProbablyDuplicateElements (int minLength, int maxLength) {
		int length = StdRandom.uniform (minLength, maxLength);
		int range = (int) Math.sqrt (length);
		double[] result = new double[length];
		for (int i=0; i<length; i++) {
			result[i] = StdRandom.uniform (-range,range);
		}
		return result;
	}
	public static double[] randomDoubleArrayUniqueElements (int minLength, int maxLength, int minVal, int maxVal) {
		if (maxLength > (maxVal - minVal))
			throw new Error ();
		int length = StdRandom.uniform (minLength, maxLength);
		double[] result = new double[length];
		for (int i=0; i<length; i++) {
			boolean foundNew = false;
			double candidate;
			do {
				candidate = StdRandom.uniform (minVal,maxVal);
				foundNew = true;
				for (int j=i-1; j>=0; j--) {
					if (result[j]==candidate)
						foundNew = false;
				}
			} while (!foundNew);
			result[i] = candidate;
		}
		return result;
	}
	public static double[] randomDoubleArrayWithLikelyItem (double item, double probabilityOfItem, int minLength, int maxLength, int minVal, int maxVal) {
		int length = StdRandom.uniform (minLength, maxLength);
		double[] result = new double[length];
		for (int i=0; i<length; i++) {
			result[i] = (StdRandom.uniform () <= probabilityOfItem) ? item : StdRandom.uniform (minVal,maxVal);
		}
		return result;
	}
	public static double[] randomDoubleArray (int minLength, int maxLength, int minVal, int maxVal) {
		int length = StdRandom.uniform (minLength, maxLength);
		double[] result = new double[length];
		for (int i=0; i<length; i++) {
			result[i] = StdRandom.uniform (minVal,maxVal);
		}
		return result;
	}
	public static double[] randomDoubleArraySorted (int minLength, int maxLength, int minVal, int maxVal) {
		double[] result = randomDoubleArray (minLength, maxLength, minVal, maxVal);
		Arrays.sort (result);
		return result;
	}
	public static String objectToString (Object object) {
		if (object == null)
			return "null";
		if (! object.getClass().isArray()) {
			return object.toString();
		} else {
			if (object instanceof Object[]) {
				return (Arrays.toString ((Object[]) object));
			} else if (object instanceof int[]) {
				return (Arrays.toString ((int[]) object));
			} else if (object instanceof double[]) {
				return (Arrays.toString ((double[]) object));
			} else if (object instanceof float[]) {
				return (Arrays.toString ((float[]) object));
			} else if (object instanceof long[]) {
				return (Arrays.toString ((long[]) object));
			} else if (object instanceof char[]) {
				return (Arrays.toString ((char[]) object));
			} else if (object instanceof boolean[]) {
				return (Arrays.toString ((boolean[]) object));
			} else if (object instanceof byte[]) {
				return (Arrays.toString ((byte[]) object));
			} else if (object instanceof short[]) {
				return (Arrays.toString ((short[]) object));
			}
		}
		throw new Error ();
	}

	public boolean checkExceptions = true;
	public int numWrongToShow;
	public final double value;
	public final String name;
	public final Collection<TG> gs;
	public final Function<TE, TR> e2r;
	public final Function<TA, TR> a2r;
	public abstract TE g2e (TG g);
	public abstract TA g2a (TG g);
	public String checkInvariants (TA a) { return null; }
	public boolean eaEquals (TE e, TA a) { return e2String (e).equals (a2String (a)); }
	public String g2String (TG g) { return String.format ("%s(%s)", name, g); };
	public String a2String (TA a) { return YTest.objectToString (a); }
	public String e2String (TE e) { return YTest.objectToString (e); }
	/**
	 * Note that g2e and g2a are abstract.
	 * @param numWrongToShow
	 * @param value
	 * @param name
	 * @param gs inputs for g2e and g2a
	 * @param expected e2r
	 * @param actual a2r
	 */
	public YTest(int numWrongToShow, double value, String name, Collection<TG> gs, Function<TE, TR> expected, Function<TA, TR> actual) {
		this.numWrongToShow = numWrongToShow;
		this.value = value;
		this.name = name;
		this.gs = gs;
		this.e2r = expected;
		this.a2r = actual;
	}
	/**
	 * @param numWrongToShow 
	 * @param value in points
	 * @param name
	 * @param gs input values
	 * @param expected way to compute expected value from g
	 * @param actual way to compute actual value from g
	 */
	public YTest(int numWrongToShow, double value, String name, Collection<TG> gs, Consumer<TE> expected, Consumer<TA> actual) {
		this (numWrongToShow, value, name, gs, x -> { expected.accept (x); return null; }, x -> { actual.accept (x); return null; });
	}
	public int maxPossibleScore () {
		return gs.size();
	}
	public int numWrong () {
		timedOut = false;
		int result = 0;
		for (TG arg : gs) {
			if (testFailed (result < numWrongToShow, arg)) {
				result += 1;
			}
		}

		if (numWrongToShow>0 && result>numWrongToShow)
			System.err.format ("  ...\n");
		return result;
	}
	int timeoutInMilliseconds = 1000;
	private boolean timedOut = false;
	private boolean testFailed (boolean showWhenWrong, TG arg) {		
		ExecutorService pool = Executors.newSingleThreadExecutor();
		if (timedOut)
			return true;
		TE eArg = g2e (arg);
		TA aArg = g2a (arg);

		TR expectedValue = null;
		Throwable expectedException = null;
		try {
			expectedValue = e2r.apply (eArg);
		} catch (Throwable e) {
			expectedException = e;
		}
		TR actualValue = null;
		Throwable actualException = null;

		Callable<TR> task = () -> a2r.apply (aArg);
		Future<TR> future = pool.submit(task);
		try {
			if (DEBUG) System.err.format ("  Start %s:\n", g2String (arg));
			actualValue = future.get(timeoutInMilliseconds, TimeUnit.MILLISECONDS);
			if (DEBUG) System.err.format ("    actualValue = %s\n", actualValue);
		} catch (TimeoutException e) {
			//if (DEBUG || showWhenWrong) {
				System.err.format ("  Timeout when executing %s: aborted after %.3f seconds:", g2String (arg), timeoutInMilliseconds/1000.0);
				System.err.format ("%scanceled\n", future.cancel(true) ? "" : "not ");
			//}
			timedOut = true;
			return true;
		} catch (InterruptedException e) {
			System.err.format ("Interrupted!!!!!\n");
			e.printStackTrace ();
			future.cancel(true);
			throw new Error ("YIKES: " +  e);
		} catch (ExecutionException e) {
			if (DEBUG) e.printStackTrace ();
			actualException = e.getCause ();
		} finally {
			if (DEBUG) System.err.format ("  Stop %s:\n", g2String (arg));
			future.cancel (true);
			pool.shutdown ();
			pool.shutdownNow ();
			// this doesn't work...
//			try {
//				if (!pool.awaitTermination(timeoutInMilliseconds, TimeUnit.MILLISECONDS)) {
//
//					System.err.format ("Thread did not stop: had to shutdown virtual machine.\n");
//					System.exit (0);
//				}
//			} catch (InterruptedException e) {
//				System.err.format ("Interrupted!!!!!\n");
//				e.printStackTrace ();
//				future.cancel(true);
//				throw new Error ("YIKES: " +  e);
//			}
		}
		String invariantString = checkInvariants (aArg);
		if (invariantString != null) {
			if (showWhenWrong) {
				System.err.format ("  Invariant error on %s:\n", g2String (arg));
				System.err.format ("    %s\n", invariantString);
			}
			return true;	
		}
		if (!resultExceptionEquals (expectedException, actualException)) {
			if (showWhenWrong) {
				System.err.format ("  Error on %s %s:\n", name, g2String (arg));
				System.err.format ("    Expected exception: %s\n", (expectedException == null) ? "none" : YTest.objectToString (expectedException));
				System.err.format ("    Actual exception:   %s\n", YTest.objectToString (actualException));
			}
			return true;
		}
		if (!resultValueEquals (expectedValue, actualValue)) {
			if (showWhenWrong) {
				System.err.format ("  Error on %s %s:\n", name, g2String (arg));
				System.err.format ("    Expected: %s\n", YTest.objectToString (expectedValue));
				System.err.format ("    Actual:   %s\n", YTest.objectToString (actualValue));
			}
			return true;
		}
		if (!eaEquals (eArg, aArg)) {
			if (showWhenWrong) {
				System.err.format ("  Error on %s %s:\n", name, g2String (arg));
				System.err.format ("    Expected: %s\n", e2String (eArg));
				System.err.format ("    Actual:   %s\n", a2String (aArg));
			}
			return true;		
		}
		return false;
	}
	public boolean resultExceptionEquals (Throwable expected, Throwable actual) {
		if (expected == null) {
			return (actual == null);
		} else {
			if (actual == null) {
				return false;
			} else {
				return expected.getClass () == actual.getClass ();
			}
		}
	}
	public boolean resultValueEquals (TR expected, TR actual) {
		if (expected == null) {
			return (actual == null);
		} else if (expected instanceof Double) {
			if (!(actual instanceof Double))
				return false;
			double e = ((Double) expected).doubleValue ();
			double a = ((Double) actual).doubleValue ();
			if (e == a)
				return true;
			double diff = Math.abs(e - a);
			return (diff < 1E-7);
		} else if (expected instanceof Float) {
			if (!(actual instanceof Float))
				return false;
			double e = ((Float) expected).doubleValue ();
			double a = ((Float) actual).doubleValue ();
			if (e == a)
				return true;
			double diff = Math.abs(e - a);
			return (diff < 1E-7);
		} else if (! expected.getClass().isArray()) {
			return expected.equals(actual);
		} else {
			if (expected instanceof Object[]) {
				if (!(actual instanceof Object[]))
					return false;
				return (Arrays.deepEquals ((Object[]) expected, (Object[]) actual));
			} else if (expected instanceof int[]) {
				if (!(actual instanceof int[]))
					return false;
				return (Arrays.equals ((int[]) expected, (int[]) actual));
			} else if (expected instanceof double[]) {
				if (!(actual instanceof double[]))
					return false;
				return (Arrays.equals ((double[]) expected, (double[]) actual));
			} else if (expected instanceof float[]) {
				if (!(actual instanceof float[]))
					return false;
				return (Arrays.equals ((float[]) expected, (float[]) actual));
			} else if (expected instanceof long[]) {
				if (!(actual instanceof long[]))
					return false;
				return (Arrays.equals ((long[]) expected, (long[]) actual));
			} else if (expected instanceof char[]) {
				if (!(actual instanceof char[]))
					return false;
				return (Arrays.equals ((char[]) expected, (char[]) actual));
			} else if (expected instanceof boolean[]) {
				if (!(actual instanceof boolean[]))
					return false;
				return (Arrays.equals ((boolean[]) expected, (boolean[]) actual));
			} else if (expected instanceof byte[]) {
				if (!(actual instanceof byte[]))
					return false;
				return (Arrays.equals ((byte[]) expected, (byte[]) actual));
			} else if (expected instanceof short[]) {
				if (!(actual instanceof short[]))
					return false;
				return (Arrays.equals ((short[]) expected, (short[]) actual));
			}
		}
		throw new Error ();
	}
}
